/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucPerfDataTable_access.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                  1) Getting the data context pointer & loop context
 *                  pointer
 *                  2) User-defined data access functions (per column) for data
 *                  in table adslAtucPerfDataTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 * Subramani:1)Return type of get_adslAtucPerfValidIntervals ,
 * get_adslAtucPerfInvalidIntervals and get_adslAtucPerfPrev1DayMoniSecs
 * changed from long to int
 *       2) Data type of Static variable long_ret changed from long to u_int32
 *
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAtucPerfDataTable_access.h"
#ifdef STUB_IOCTL
#include "adslAtucPerfDataTable_enums.h"
#endif


/* column number definitions for table adslAtucPerfDataTable */
#include "adslAtucPerfDataTable_columns.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/

// *** Global variable definitions
static u_int32 long_ret;

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** returns the first data point within the adslAtucPerfDataTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAtucPerfDataTable_get_first_data_point(void **my_loop_context,
                                           void **my_data_context,
                                           netsnmp_variable_list *
                                           put_index_data,
                                           netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-First: adslAtucPerfDataTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }

    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAtucPerfDataTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAtucPerfDataTable_get_next_data_point(void **my_loop_context,
                                          void **my_data_context,
                                          netsnmp_variable_list *
                                          put_index_data,
                                          netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-Next: adslAtucPerfDataTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

     put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/*
 * User-defined data access functions (per column) for data in table
 * adslAtucPerfDataTable
 */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */

#if defined(HAVE_OPTIMIZED_CODE)

u_int32  *
get_adslAtucPerfDataLoss(void *data_context, size_t * ret_len,
			unsigned int var)
{
  unsigned int flags = 0;
  struct atucPerfDataEntry atucPerfDataEntryStruct;
  memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
  atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
  
  switch (var)  {

	case COLUMN_ADSLATUCPERFLOFS:
	   flags = ATUC_PERF_LOFS_FLAG;
	   break;

	case COLUMN_ADSLATUCPERFLOSS:
	   flags = ATUC_PERF_LOSS_FLAG;
	   break;

	case COLUMN_ADSLATUCPERFESS:
	   flags = ATUC_PERF_ESS_FLAG;
	   break;
	   
	case COLUMN_ADSLATUCPERFINITS:
	   flags = ATUC_PERF_INITS_FLAG;
	   break;

     	default:
     	   DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
     	   return NULL;
  }

  SET_FLAG(&atucPerfDataEntryStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
            (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags, flags) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataTable: GET- %x CMV fail\n", flags));

    switch (flags)  {
	    
	case ATUC_PERF_LOFS_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfLofs;
	   break;

	case ATUC_PERF_LOSS_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfLoss;
	   break;

	case ATUC_PERF_ESS_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfESs;
	   break;
		
	case ATUC_PERF_INITS_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfInits;
	   break;

        default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
    }
    
    CLR_FLAG(&atucPerfDataEntryStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}


int *
get_adslAtucPerfIntervalData(void *data_context, size_t * ret_len,
			unsigned int var)
{
  unsigned int flags = 0;
  struct atucPerfDataEntry atucPerfDataEntryStruct;
  memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
  atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);

  switch (var) {

      case COLUMN_ADSLATUCPERFVALIDINTERVALS:
	 flags = ATUC_PERF_VALID_INTVLS_FLAG;
	 break;
	 
      case COLUMN_ADSLATUCPERFINVALIDINTERVALS:
	 flags = ATUC_PERF_INVALID_INTVLS_FLAG;
	 break;

      case COLUMN_ADSLATUCPERFPREV1DAYMONISECS:
	 flags = ATUC_PERF_PREV_1DAY_MON_SEC_FLAG;
	 break;
	 
      default:
     	 DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
     	 return NULL;
 }
  
  SET_FLAG(&atucPerfDataEntryStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
            (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags, flags) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataTable: GET- %x CMV fail\n", flags));

    switch (flags)  {
	    
	case ATUC_PERF_VALID_INTVLS_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfValidIntervals;
	   break;

	case ATUC_PERF_INVALID_INTVLS_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfInvalidIntervals;
	   break;

	case ATUC_PERF_PREV_1DAY_MON_SEC_FLAG:
    	   long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayMoniSecs;
	   break;

        default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
    }
    
    CLR_FLAG(&atucPerfDataEntryStruct.flags, flags);
    *ret_len = sizeof(int);
    return (int *) &long_ret;
}


u_int32  *
get_adslAtucPerfDataStats(void *data_context, size_t * ret_len,
			unsigned int var)
{
  unsigned int flags = 0;
  struct atucPerfDataEntry atucPerfDataEntryStruct;
  memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
  atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);

  switch (var) {

      case COLUMN_ADSLATUCPERFCURR15MINTIMEELAPSED:
	 flags = ATUC_PERF_CURR_15MIN_TIME_ELAPSED_FLAG; 
         break;
	 
      case COLUMN_ADSLATUCPERFCURR15MINLOFS:
	 flags = ATUC_PERF_CURR_15MIN_LOFS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR15MINLOSS:
	 flags = ATUC_PERF_CURR_15MIN_LOSS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR15MINESS:
	 flags = ATUC_PERF_CURR_15MIN_ESS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR15MININITS:
	 flags = ATUC_PERF_CURR_15MIN_INIT_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR1DAYTIMEELAPSED:
	 flags = ATUC_PERF_CURR_1DAY_TIME_ELAPSED_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR1DAYLOFS:
	 flags = ATUC_PERF_CURR_1DAY_LOFS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR1DAYLOSS:
	 flags = ATUC_PERF_CURR_1DAY_LOSS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR1DAYESS:
	 flags = ATUC_PERF_CURR_1DAY_ESS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFCURR1DAYINITS:
	 flags = ATUC_PERF_CURR_1DAY_INIT_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFPREV1DAYLOFS:
	 flags = ATUC_PERF_PREV_1DAY_LOFS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFPREV1DAYLOSS:
	 flags = ATUC_PERF_PREV_1DAY_LOSS_FLAG;
	 break;
	  
      case COLUMN_ADSLATUCPERFPREV1DAYESS:
	 flags = ATUC_PERF_PREV_1DAY_ESS_FLAG;
	 break;
	 
      case COLUMN_ADSLATUCPERFPREV1DAYINITS:
	 flags = ATUC_PERF_PREV_1DAY_INITS_FLAG;
	 break;
	  
      default:
     	 DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
         return NULL;
      }

  SET_FLAG(&atucPerfDataEntryStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
            (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags, flags) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataTable: GET- %x CMV fail\n", flags));

     switch (flags)  {

	 case ATUC_PERF_CURR_15MIN_TIME_ELAPSED_FLAG:
	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinTimeElapsed;
            break;
	 
	 case ATUC_PERF_CURR_15MIN_LOFS_FLAG:
	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinLofs;
	    break;
	  
	 case ATUC_PERF_CURR_15MIN_LOSS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinLoss;
	    break;
	  
	 case ATUC_PERF_CURR_15MIN_ESS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinESs;
	    break;
	  
	 case ATUC_PERF_CURR_15MIN_INIT_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinInits;
	    break;
	  
	 case ATUC_PERF_CURR_1DAY_TIME_ELAPSED_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayTimeElapsed;
	    break;
	  
	 case ATUC_PERF_CURR_1DAY_LOFS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayLofs;
	    break;
	  
	 case ATUC_PERF_CURR_1DAY_LOSS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayLoss;
	    break;
	  
	 case ATUC_PERF_CURR_1DAY_ESS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayESs;
	    break;
	  
	 case ATUC_PERF_CURR_1DAY_INIT_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayInits;
            break;
	  
	 case ATUC_PERF_PREV_1DAY_LOFS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayLofs;
	    break;
	  
	 case ATUC_PERF_PREV_1DAY_LOSS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayLoss;
	    break;
	  
	 case ATUC_PERF_PREV_1DAY_ESS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayESs;
	    break;
	 
	 case ATUC_PERF_PREV_1DAY_INITS_FLAG:
    	    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayInits;
	    break;
	  
         default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
     }
     
    CLR_FLAG(&atucPerfDataEntryStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}

#else

/** XXX: return a data pointer to the data for the adslAtucPerfLofs column
    and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfLofs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_LOFS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags, ATUC_PERF_LOFS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataLOFS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_LOFS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfLofs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfLofs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfLoss column
    and set ret_len to its proper size in bytes. */

u_int32         *
get_adslAtucPerfLoss(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_LOSS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags, ATUC_PERF_LOSS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataLOSS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_LOSS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfLoss;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfLoss);
    return (u_int32 *) &long_ret;
}


/** XXX: return a data pointer to the data for the adslAtucPerfESs column and
    set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfESs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_ESS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags, ATUC_PERF_ESS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataESS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_ESS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfESs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfESs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfInits column
    and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfInits(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_INITS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if(IS_FLAG_SET(&atucPerfDataEntryStruct.flags, ATUC_PERF_INITS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataInit CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_INITS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfInits;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfInits);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfValidIntervals
    column and set ret_len to its proper size in bytes. */
int           *
get_adslAtucPerfValidIntervals(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_VALID_INTVLS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
             ATUC_PERF_VALID_INTVLS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataValidIntvl CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_VALID_INTVLS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfValidIntervals;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfValidIntervals);
    return (int *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfInvalidIntervals
    column and set ret_len to its proper size in bytes. */
int           *
get_adslAtucPerfInvalidIntervals(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_INVALID_INTVLS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
            ATUC_PERF_INVALID_INTVLS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataInvalidIntvl CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_INVALID_INTVLS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfInvalidIntervals;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfInvalidIntervals);
    return (int *) &long_ret;
}

/** XXX: return a data pointer to the data for the
    adslAtucPerfCurr15MinTimeElapsed column and set ret_len to its proper size
    in bytes. */
u_int32         *
get_adslAtucPerfCurr15MinTimeElapsed(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_15MIN_TIME_ELAPSED_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
            ATUC_PERF_CURR_15MIN_TIME_ELAPSED_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr15Min CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags,
            ATUC_PERF_CURR_15MIN_TIME_ELAPSED_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinTimeElapsed;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr15MinTimeElapsed);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr15MinLofs
    column and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucPerfCurr15MinLofs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_LOFS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_15MIN_LOFS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr15MinLOF CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_LOFS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinLofs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr15MinLofs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr15MinLoss
    column and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucPerfCurr15MinLoss(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_LOSS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_15MIN_LOSS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr15MinLOSS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_LOSS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinLoss;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr15MinLoss);
    return (u_int32 *) &long_ret;
}


/** XXX: return a data pointer to the data for the adslAtucPerfCurr15MinESs
    column and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucPerfCurr15MinESs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_ESS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
            ATUC_PERF_CURR_15MIN_ESS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr15MinESS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_ESS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinESs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr15MinESs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr15MinInits
    column and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucPerfCurr15MinInits(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_INIT_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_15MIN_INIT_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr15MinInits CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_15MIN_INIT_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr15MinInits;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr15MinInits);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the
    adslAtucPerfCurr1DayTimeElapsed column and set
         ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfCurr1DayTimeElapsed(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_1DAY_TIME_ELAPSED_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_1DAY_TIME_ELAPSED_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr1DayElapse CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_1DAY_TIME_ELAPSED_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayTimeElapsed;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr1DayTimeElapsed);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr1DayLofs
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfCurr1DayLofs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_LOFS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_1DAY_LOFS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr1DayLOF CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_LOFS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayLofs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr1DayLofs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr1DayLoss
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfCurr1DayLoss(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_LOSS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_1DAY_LOSS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr1DayLOSS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_LOSS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayLoss;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr1DayLoss);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr1DayESs
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfCurr1DayESs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_ESS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
         ATUC_PERF_CURR_1DAY_ESS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr1DayESS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_ESS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayESs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr1DayESs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfCurr1DayInits
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfCurr1DayInits(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_INIT_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_CURR_1DAY_INIT_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataCurr1DayInit CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_CURR_1DAY_INIT_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfCurr1DayInits;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfCurr1DayInits);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfPrev1DayMoniSecs
    column and set ret_len to its proper size in bytes. */
int           *
get_adslAtucPerfPrev1DayMoniSecs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_MON_SEC_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
            ATUC_PERF_PREV_1DAY_MON_SEC_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataPrev1DayMonSec CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_MON_SEC_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayMoniSecs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfPrev1DayMoniSecs);
    return (int *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfPrev1DayLofs
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfPrev1DayLofs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_LOFS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
            ATUC_PERF_PREV_1DAY_LOFS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataPrev1DayLOF CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_LOFS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayLofs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfPrev1DayLofs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfPrev1DayLoss
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfPrev1DayLoss(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_LOSS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_PREV_1DAY_LOSS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataPrev1DayLOSS CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_LOSS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayLoss;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfPrev1DayLoss);
    return (u_int32 *) &long_ret;
}


/** XXX: return a data pointer to the data for the adslAtucPerfPrev1DayESs
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfPrev1DayESs(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_ESS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
         ATUC_PERF_PREV_1DAY_ESS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataPrev1DayESS CMV fail\n"));
       
    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_ESS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayESs;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfPrev1DayESs);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucPerfPrev1DayInits
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucPerfPrev1DayInits(void *data_context, size_t * ret_len)
{
    struct atucPerfDataEntry atucPerfDataEntryStruct;

    memset(&atucPerfDataEntryStruct,0,sizeof(atucPerfDataEntry));
    atucPerfDataEntryStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_INITS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_PERF_DATA, (void *) &atucPerfDataEntryStruct);
#else
    get_adslAtucPerfDataTable(Fd, GET_ADSL_ATUC_PERF_DATA,
        (void *) &atucPerfDataEntryStruct);
#endif
    if (IS_FLAG_SET(&atucPerfDataEntryStruct.flags,
        ATUC_PERF_PREV_1DAY_INITS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "PerfDataPrev1DayInit CMV fail\n"));

    CLR_FLAG(&atucPerfDataEntryStruct.flags, ATUC_PERF_PREV_1DAY_INITS_FLAG);
    long_ret = atucPerfDataEntryStruct.adslAtucPerfPrev1DayInits;
    *ret_len = sizeof(atucPerfDataEntryStruct.adslAtucPerfPrev1DayInits);
    return (u_int32 *) &long_ret;
}
#endif


#ifndef ENABLE_DRIVER_CALL
int get_adslAtucPerfDataTable(int Fd2,int GET_adsl_ATUC_PERF_DATA2,
        void *atucPerfDataValue)
{

    struct atucPerfDataEntry *atucPerfData;
    atucPerfData = (atucPerfDataEntry *) atucPerfDataValue;
    atucPerfData->adslAtucPerfLofs = 10;
    atucPerfData->adslAtucPerfLoss = 20;
    atucPerfData->adslAtucPerfESs = 30;
    atucPerfData->adslAtucPerfInits = 40;
    atucPerfData->adslAtucPerfValidIntervals = 10;
    atucPerfData->adslAtucPerfInvalidIntervals = 20;
    atucPerfData->adslAtucPerfCurr15MinTimeElapsed = 10;
    atucPerfData->adslAtucPerfCurr15MinLofs = 15;
    atucPerfData->adslAtucPerfCurr15MinLoss = 20;
    atucPerfData->adslAtucPerfCurr15MinESs = 25;
    atucPerfData->adslAtucPerfCurr15MinInits = 30;
    atucPerfData->adslAtucPerfCurr1DayTimeElapsed = 40;
    atucPerfData->adslAtucPerfCurr1DayLofs = 50;
    atucPerfData->adslAtucPerfCurr1DayLoss = 60;
    atucPerfData->adslAtucPerfCurr1DayESs = 65;
    atucPerfData->adslAtucPerfCurr1DayInits = 70;
    atucPerfData->adslAtucPerfPrev1DayMoniSecs = 80;
    atucPerfData->adslAtucPerfPrev1DayLofs = 85;
    atucPerfData->adslAtucPerfPrev1DayLoss = 90;
    atucPerfData->adslAtucPerfPrev1DayESs = 95;
    atucPerfData->adslAtucPerfPrev1DayInits = 100;

    return 0;
}
#endif
