/************************************************************************
 *
 *    Copyright (c) 2005
 *    Infineon Technologies AG
 *    St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 *    This document contains proprietary information belonging to Infineon
 *    Technologies AG. Passing on and copying of this document, and communication
 *    of its contents is not permitted without prior written authorisation.
 *
 *    --------------------------------------------------------------------
 *    
 *        Project : INCA-IP
 *        Block   : IFX MIB
 *        Creator : Sharon Tseng
 *        File    : ifxMIB_common.h
 *        Abstract: 
 *                  
 *        Date    : 18-Jul-2005
 *        Ref. Doc: IIP-DFS.doc
 *    
 *    Modification History:
 *           By              Date     Ver.   Modification Description
 *           --------------- -------- -----  -----------------------------
 *
 ************************************************************************/
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <memory.h>
#include <linux/if.h>
#include "ifxMIB_common.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/
STATIC int16                    nCmFifoFd = -1, nMgtFifoFd = -1, nMgtId;
x_IFX_CM_AppAddressBookEntry    axPBEntry[IFX_WEB_MAX_PB_ENTRIES];
int16                           nNoOfPbEntries;
IFX_MIB_MSG_S                   xMibMsg;
IFX_MIB_MSG_S                   xPrevMibMsg;
RTPSESSIONENTRY_S               **pstRtpSessCfg = NULL;
RTPTXENTRY_S                    **pstRtpTxCfg = NULL;
RTPRXENTRY_S                    **pstRtpRxCfg = NULL;
DNSCFG_S                        **pstWanDnsCfg = NULL;
PPPOECFG_S                      **pstWanPppoeCfg = NULL;
STATICCFG_S                     **pstWanStaticCfg = NULL;
QOSEXTENDCFG_S                  **pstQosExtendCfg = NULL;
QOSSWENHANCECFG_S               **pstQosSwEnCfg = NULL;
QOSSTATICMACCFG_S               **pstQosStaticMacCfg = NULL;
VOIPCODECCFG_S                  **pstVoipCodec = NULL;

/*
 * ===========================================================================
 *                           EXPORTED FUNCTIONS
 * ===========================================================================
 */

/* This is used, because the ADSL-MIB augments ifTable */
extern unsigned char *var_ifEntry(struct variable *,
                                  oid *, size_t *,
                                  int, size_t *, WriteMethod **);
extern  unsigned char *var_interfaces(struct variable * ,
                                oid * ,size_t * ,
                                int , size_t * , WriteMethod ** );


/*
 * ===========================================================================
 *                          GLOBAL FUNCTIONS
 * ===========================================================================
*/
/*** XXX Return the IfType of an IfEntry in the IfTable XXX ***/
int
ifx_GetType(int index)
{
    oid             name[MAX_OID_LEN] = { 1, 3, 6, 1, 2, 1, 2, 2, 1, 3 };
    size_t          length = 10;
    struct variable ifType_variable =
        { 3, ASN_INTEGER, RONLY, var_ifEntry, 10,
        {1, 3, 6, 1, 2, 1, 2, 2, 1, 3}
    };
    unsigned char  *p;
    size_t          var_len;
    WriteMethod    *write_method;

    name[length] = index;
    length++;

    p = var_ifEntry(&ifType_variable,
                    name, &length,
                    1 /* exact */ , &var_len, &write_method);
    if (!p)
        return 0;

    return *(int *) p;
}

/*** XXX Return the IfAdminStatus of an IfEntry in the IfTable XXX ***/
int
ifx_GetAdminStatus(int index)
{
    oid             name[MAX_OID_LEN] = { 1, 3, 6, 1, 2, 1, 2, 2, 1, 7 };
    size_t          length = 10;
    struct variable ifType_variable =
        { 7, ASN_INTEGER, RONLY, var_ifEntry, 10,
        {1, 3, 6, 1, 2, 1, 2, 2, 1, 7}
    };
    unsigned char  *p;
    size_t          var_len;
    WriteMethod    *write_method;

    name[length] = index;
    length++;

    p = var_ifEntry(&ifType_variable,
                    name, &length,
                    1 /* exact */ , &var_len, &write_method);
    if (!p)
        return 0;

    return *(int *) p;
}

/*** XXX Return the Number of interfaces in the IfTable XXX ***/
int
ifx_GetNoInterfaces()
{
    oid             name[MAX_OID_LEN] = { 1, 3, 6, 1, 2, 1, 2, 2, 1, 0 };
    size_t          length = 10;
    struct variable ifType_variable =
        { 0, ASN_INTEGER, RONLY, var_ifEntry, 10,
        {1, 3, 6, 1, 2, 1, 2, 2, 1, 0}
    };
    unsigned char  *p;
    size_t          var_len;
    WriteMethod    *write_method;

    name[length] = 0;
    length++;

    p = var_interfaces(&ifType_variable,
                    name, &length,
                    1 /* exact */ , &var_len, &write_method);
    if (!p)
        return 0;

    return *(int *) p;
}

/*******************************************************************************
 *  Function Name : IFX_ValidateIpAddress
 *  Description   :This function Check the IP Address.
 *  Invoked Fns   :
 *  Input Values  :char *
 *  Output Values :
 *  Return Value  : void
 *  Note          :
 * ******************************************************************************/
char
IFX_ValidateIpAddress (unsigned char *pszBuff)
{
  int iCount = 0, iDotCount = 0,iCount1=0;
  char acTemp[4];
  while (*pszBuff != '\0')
  {
    if (*pszBuff == '.')
    {
      iDotCount++;
      acTemp[iCount] = '\0';
      if ((iCount > 0) && (atoi (acTemp) < 256))
      {
		if (iCount1 == 0 && atoi (acTemp) < 1)
		{
		  return -1;
		}
        memset (acTemp, '\0', 4);
        iCount = 0;
		iCount1++;
        pszBuff++;
      }
      else
      {
        return -1;
      }
    }
    else
    {
      if (!isdigit (*pszBuff))
      {
        return -1;
      }
      acTemp[iCount++] = *pszBuff;
      pszBuff++;
      if (iCount > 3)
      {
        return -1;
      }
    }
  }
  if (iCount < 1 || atoi (acTemp) > 255 || iDotCount != 3)
  {
    return -1;
  }
  return 0;
}

/*******************************************************************************
 *  Function Name : IFX_ValidateMacAddress
 *  Description   :This function Check the MAC Address.
 *  Invoked Fns   :
 *  Input Values  :char *
 *  Output Values :
 *  Return Value  : void
 *  Note          :
 * ******************************************************************************/
int
IFX_ValidateMacAddress(unsigned char *pszBuff)
{
    int iCount = 0, iDashCount = 0;
    char acTemp[3];

    while (*pszBuff != '\0')
    {
        if (*pszBuff == '-')
        {
            iDashCount++;
            acTemp[iCount] = '\0';
            
            memset (acTemp, '\0', 3);
            iCount = 0;
            pszBuff++;
        }
        else
        {
            if (!isxdigit (*pszBuff))
            {
                return -1;
            }

            acTemp[iCount++] = *pszBuff;
            pszBuff++;
            if (iCount > 2)
            {
                return -1;
            }
        }
    }

    if (iCount < 1 || iDashCount != 5)
    {
        return -1;
    }
    return 0;
}

/*******************************************************************************
 *  Function Name : ifx_strToHex
 *  Description   :This function Check the string is HEX or not.
 *  Invoked Fns   :
 *  Input Values  :char *
 *  Output Values :
 *  Return Value  : void
 *  Note          :
 * ******************************************************************************/
int
ifx_strToHex(char *string)
{
    char c1;
    int i, len=0, var=0, shift=0;

    if (string == 0)
        return -1;

    len = strlen(string);
    if (string[len] != 0)
        return -1;

    for(i = 0; i<len; i++)
    {
      if (!isxdigit(string[i]))
        return -1;
      c1 = string[i];
      c1 = tolower(c1);
      c1 = (c1 < 'a')? (c1 - '0') : (c1 - 'a' + 10);
      shift = len - i - 1; 

      // Ex: 1FF = 1*16*16 + 15*16 + 15 
      if (shift)
        var += c1<<4*shift;
      else
        var += c1;
    }

    return 0;
}

/*****************************************************************************
* Function Name: websSetCfgData
* Description  : set configuration data
* Input Value  : pFileName       ==> Open File Name
*                pTag            ==> TAG_xxx
*                nDataCount      ==> Data number for setting.
*                pData           ==> Data content for setting.
* Output Value :
* Return Value : 0:Fail 1: Success
* Notes :
*****************************************************************************/
int
websSetCfgData (const char_t * pFileName, const char_t * pTag, int nDataCount,
                   const char_t* pData, ... )
{
	FILE   	*fdIn, *fdOut;
	char_t 	arrInLine[MAX_FILELINE_LEN];
	char_t	strTempName[MAX_FILENAME_LEN];
	char_t	buf[BUF_SIZE_50K];	
	char_t 	arrBeginTag[MAX_TAG_LEN], arrEndTag[MAX_TAG_LEN];
	bool_t 	bTagArea = FALSE;
	va_list	args;

        /* First, extract data for list and store into buffer*/
    	va_start( args, pData );
	gstrcpy(buf, pData);
	
	
    	while( --nDataCount )
    	{    	
 		pData = va_arg(args, char_t *);
		gstrcat(buf, pData);
	}
    	va_end(args);

	memset(arrBeginTag, 0x00, MAX_TAG_LEN);
	memset(arrEndTag, 0x00, MAX_TAG_LEN);
    	gsprintf(arrBeginTag, "#<< %s", pTag);
    	gsprintf(arrEndTag, "#>> %s", pTag);    
	
	if ((fdIn = gfopen(pFileName, "r+t")) == NULL)
	{
		fclose(fdIn);
		return 0;
	}

	gsprintf(strTempName, T("%s.tmp"), pFileName);
	if ((fdOut = gfopen(strTempName, "w+")) == NULL)
	{
		fclose(fdOut);
		return 0;
	}

	while (gfgets(arrInLine, sizeof(arrInLine), fdIn))    
    	{
    	    	if (!gstrlen (arrInLine))
    	    	    	continue;  /*Null line*/

        	/* remove "\n" and keep string only*/
        	arrInLine[gstrlen(arrInLine)-1] = 0; 
        
        	if (gstrncmp(arrInLine, arrBeginTag, sizeof(arrBeginTag)) == 0) 
        	{
	        	bTagArea = TRUE;
        	}
        
       		gfprintf(fdOut, "%s\n", arrInLine);        
       	
		if (bTagArea)
		{
        		/* Print all datas to file*/
			int ret;
			ret = gfprintf(fdOut, buf);			

			while (gfgets(arrInLine, sizeof(arrInLine), fdIn))
			{
				arrInLine[gstrlen(arrInLine)-1] = 0;
		        	if (gstrncmp(arrInLine, arrEndTag, sizeof(arrEndTag)) == 0)
		        	{
			       		gfprintf(fdOut, "%s\n", arrInLine);
					bTagArea = FALSE;
					break;
				}
			}			
		}        
    	}

	fclose(fdIn);
	fclose(fdOut);

	if (gremove(pFileName) == 0)
	{
		grename(strTempName, pFileName);	
	}
	return 1;
}

int  
ifx_MIB_Dalloc()
{
    int i = 0;

    if (xMibMsg.nType != xPrevMibMsg.nType ||
        xMibMsg.nCount != xPrevMibMsg.nCount ||
        xMibMsg.nPSize != xPrevMibMsg.nPSize ||
        xMibMsg.nSize != xPrevMibMsg.nSize)
    {
        ifx_MIB_free_Dalloc();
        memcpy ((char *) &xPrevMibMsg, (char *) &xMibMsg, sizeof(IFX_MIB_MSG_S));
    }

    switch (xMibMsg.nType)
    {
        case IFX_CM_RTP_SESSION_TABLE:
            if (!pstRtpSessCfg)
            {
                pstRtpSessCfg = (RTPSESSIONENTRY_S **) calloc(xMibMsg.nCount,
                                                              xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstRtpSessCfg[i] = (RTPSESSIONENTRY_S *) malloc(xMibMsg.nSize);
                    memset (pstRtpSessCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstRtpSessCfg[%d] pointer [%d] ****\n",
                           i, pstRtpSessCfg[i]); */
                }
                /* printf("****  alloc pstRtpSessCfg pointer [%d] ****\n",
                       pstRtpSessCfg); */
            }
            break;

        case IFX_CM_RTP_TX_TABLE:
            if (!pstRtpTxCfg)
            {
                pstRtpTxCfg = (RTPTXENTRY_S **) calloc(xMibMsg.nCount,
                                                       xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstRtpTxCfg[i] = (RTPTXENTRY_S *) malloc(xMibMsg.nSize);
                    memset (pstRtpTxCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstRtpTxCfg[%d] pointer [%d] ****\n",
                           i, pstRtpTxCfg[i]); */
                }
                /* printf("****  alloc pstRtpTxCfg pointer [%d] ****\n",
                       pstRtpTxCfg); */
            }
            break;

        case IFX_CM_RTP_RX_TABLE:
            if (!pstRtpRxCfg)
            {
                pstRtpRxCfg = (RTPRXENTRY_S **) calloc(xMibMsg.nCount,
                                                       xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstRtpRxCfg[i] = (RTPRXENTRY_S *) malloc(xMibMsg.nSize);
                    memset (pstRtpRxCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstRtpRxCfg[%d] pointer [%d] ****\n",
                           i, pstRtpRxCfg[i]); */
                }
                /* printf("****  alloc pstRtpRxCfg pointer [%d] ****\n",
                       pstRtpRxCfg); */
            }
            break;

        case IFX_MIB_WAN_DNS:
            if (!pstWanDnsCfg)
            {
                pstWanDnsCfg = (DNSCFG_S **) calloc(xMibMsg.nCount,
                                                    xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstWanDnsCfg[i] = (DNSCFG_S *) malloc(xMibMsg.nSize);
                    memset (pstWanDnsCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstWanDnsCfg[%d] pointer [%d] ****\n",
                           i, pstWanDnsCfg[i]); */
                }
                /* printf("****  alloc pstWanDnsCfg pointer [%d] ****\n",
                       pstWanDnsCfg); */
            }
            break;

        case IFX_MIB_WAN_PPPOE:
            if (!pstWanPppoeCfg)
            {
                pstWanPppoeCfg = (PPPOECFG_S **) calloc(xMibMsg.nCount,
                                                        xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstWanPppoeCfg[i] = (PPPOECFG_S *) malloc(xMibMsg.nSize);
                    memset (pstWanPppoeCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstWanPppoeCfg[%d] pointer [%d] ****\n",
                           i, pstWanPppoeCfg[i]); */
                }
                /* printf("****  alloc pstWanPppoeCfg pointer [%d] ****\n",
                       pstWanPppoeCfg); */
            }
            break;

        case IFX_MIB_WAN_STATIC:
            if (!pstWanStaticCfg)
            {
                pstWanStaticCfg = (STATICCFG_S **) calloc(xMibMsg.nCount,
                                                          xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstWanStaticCfg[i] = (STATICCFG_S *) malloc(xMibMsg.nSize);
                    memset (pstWanStaticCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstWanStaticCfg[%d] pointer [%d] ****\n",
                           i, pstWanStaticCfg[i]); */
                }
                /* printf("****  alloc pstWanStaticCfg pointer [%d] ****\n",
                       pstWanStaticCfg); */
            }
            break;

        case IFX_MIB_QOS_EXTEND:
            if (!pstQosExtendCfg)
            {
                pstQosExtendCfg = (QOSEXTENDCFG_S **) calloc(xMibMsg.nCount,
                                                             xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstQosExtendCfg[i] = (QOSEXTENDCFG_S *) malloc(xMibMsg.nSize);
                    memset (pstQosExtendCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstQosExtendCfg[%d] pointer [%d] ****\n",
                           i, pstQosExtendCfg[i]); */
                }
                /* printf("****  alloc pstQosExtendCfg pointer [%d] ****\n",
                       pstQosExtendCfg); */
            }
            break;

        case IFX_MIB_QOS_SWENHANCE:
            if (!pstQosSwEnCfg)
            {
                pstQosSwEnCfg = (QOSSWENHANCECFG_S **) calloc(xMibMsg.nCount,
                                                              xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstQosSwEnCfg[i] = (QOSSWENHANCECFG_S *) malloc(xMibMsg.nSize);
                    memset (pstQosSwEnCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstQosSwEnCfg[%d] pointer [%d] ****\n",
                           i, pstQosSwEnCfg[i]); */
                }
                /* printf("****  alloc pstQosSwEnCfg pointer [%d] ****\n",
                       pstQosSwEnCfg); */
            }
            break;

        case IFX_MIB_QOS_STATICMAC:
            if (!pstQosStaticMacCfg)
            {
                pstQosStaticMacCfg = (QOSSTATICMACCFG_S **) calloc(xMibMsg.nCount,
                                                                   xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstQosStaticMacCfg[i] = (QOSSTATICMACCFG_S *) malloc(xMibMsg.nSize);
                    memset (pstQosStaticMacCfg[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstQosStaticMacCfg[%d] pointer [%d] ****\n",
                           i, pstQosStaticMacCfg[i]); */
                }
                /* printf("****  alloc pstQosStaticMacCfg pointer [%d] ****\n",
                       pstQosStaticMacCfg); */
            }
            break;

        case IFX_MIB_VOIP_CODEC:
            if (!pstVoipCodec)
            {
                pstVoipCodec = (VOIPCODECCFG_S **) calloc(xMibMsg.nCount,
                                                          xMibMsg.nPSize);
                for (i = 0; i < xMibMsg.nCount; i++)
                {
                    pstVoipCodec[i] = (VOIPCODECCFG_S *) malloc(xMibMsg.nSize);
                    memset (pstVoipCodec[i], 0x00, xMibMsg.nSize);
                    /* printf("****  alloc pstVoipCodec[%d] pointer [%d] ****\n",
                           i, pstVoipCodec[i]); */
                }
                /* printf("****  alloc pstVoipCodec pointer [%d] ****\n",
                       pstVoipCodec); */
            }
            break;

        default:
            break;
    }

    return 1;
}

int 
ifx_MIB_free_Dalloc()
{
    int i = 0;

    switch (xPrevMibMsg.nType)
    {
        case IFX_CM_RTP_SESSION_TABLE:
            if (pstRtpSessCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstRtpSessCfg[%d] pointer [%d] ****\n",
                           i, pstRtpSessCfg[i]); */
                    free(pstRtpSessCfg[i]);
                    pstRtpSessCfg[i] = NULL;
                }
                /* printf("****  free pstRtpSessCfg pointer [%d] ****\n",
                       pstRtpSessCfg); */
                free(pstRtpSessCfg);
                pstRtpSessCfg = NULL;
            }
            break;

        case IFX_CM_RTP_TX_TABLE:
            if (pstRtpTxCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstRtpTxCfg[%d] pointer [%d] ****\n",
                           i, pstRtpTxCfg[i]); */
                    free(pstRtpTxCfg[i]);
                    pstRtpTxCfg[i] = NULL;
                }
                /* printf("****  free pstRtpTxCfg pointer [%d] ****\n",
                       pstRtpTxCfg); */
                free(pstRtpTxCfg);
                pstRtpTxCfg = NULL;
            }
            break;

        case IFX_CM_RTP_RX_TABLE:
            if (pstRtpRxCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstRtpRxCfg[%d] pointer [%d] ****\n",
                           i, pstRtpRxCfg[i]); */
                    free(pstRtpRxCfg[i]);
                    pstRtpRxCfg[i] = NULL;
                }
                /* printf("****  free pstRtpRxCfg pointer [%d] ****\n",
                       pstRtpRxCfg); */
                free(pstRtpRxCfg);
                pstRtpRxCfg = NULL;
            }
            break;

        case IFX_MIB_WAN_DNS:
            if (pstWanDnsCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstWanDnsCfg[%d] pointer [%d] ****\n",
                           i, pstWanDnsCfg[i]); */
                    free(pstWanDnsCfg[i]);
                    pstWanDnsCfg[i] = NULL;
                }
                /* printf("****  free pstWanDnsCfg pointer [%d] ****\n",
                       pstWanDnsCfg); */
                free(pstWanDnsCfg);
                pstWanDnsCfg = NULL;
            }
            break;

        case IFX_MIB_WAN_PPPOE:
            if (pstWanPppoeCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstWanPppoeCfg[%d] pointer [%d] ****\n",
                           i, pstWanPppoeCfg[i]); */
                    free(pstWanPppoeCfg[i]);
                    pstWanPppoeCfg[i] = NULL;
                }
                /* printf("****  free pstWanPppoeCfg pointer [%d] ****\n",
                       pstWanPppoeCfg); */
                free(pstWanPppoeCfg);
                pstWanPppoeCfg = NULL;
            }
            break;

        case IFX_MIB_WAN_STATIC:
            if (pstWanStaticCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstWanStaticCfg[%d] pointer [%d] ****\n",
                           i, pstWanStaticCfg[i]); */
                    free(pstWanStaticCfg[i]);
                    pstWanStaticCfg[i] = NULL;
                }
                /* printf("****  free pstWanStaticCfg pointer [%d] ****\n",
                       pstWanStaticCfg); */
                free(pstWanStaticCfg);
                pstWanStaticCfg = NULL;
            }
            break;

        case IFX_MIB_QOS_EXTEND:
            if (pstQosExtendCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstQosExtendCfg[%d] pointer [%d] ****\n",
                           i, pstQosExtendCfg[i]); */
                    free(pstQosExtendCfg[i]);
                    pstQosExtendCfg[i] = NULL;
                }
                /* printf("****  free pstQosExtendCfg pointer [%d] ****\n",
                       pstQosExtendCfg); */
                free(pstQosExtendCfg);
                pstQosExtendCfg = NULL;
            }
            break;

        case IFX_MIB_QOS_SWENHANCE:
            if (pstQosSwEnCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstQosSwEnCfg[%d] pointer [%d] ****\n",
                           i, pstQosSwEnCfg[i]); */
                    free(pstQosSwEnCfg[i]);
                    pstQosSwEnCfg[i] = NULL;
                }
                /* printf("****  free pstQosSwEnCfg pointer [%d] ****\n",
                       pstQosSwEnCfg); */
                free(pstQosSwEnCfg);
                pstQosSwEnCfg = NULL;
            }
            break;

        case IFX_MIB_QOS_STATICMAC:
            if (pstQosStaticMacCfg)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstQosStaticMacCfg[%d] pointer [%d] ****\n",
                           i, pstQosStaticMacCfg[i]); */
                    free(pstQosStaticMacCfg[i]);
                    pstQosStaticMacCfg[i] = NULL;
                }
                /* printf("****  free pstQosStaticMacCfg pointer [%d] ****\n",
                       pstQosStaticMacCfg); */
                free(pstQosStaticMacCfg);
                pstQosStaticMacCfg = NULL;
            }
            break;

        case IFX_MIB_VOIP_CODEC:
            if (pstVoipCodec)
            {
                for (i = 0; i < xPrevMibMsg.nCount; i++)
                {
                    /* printf("****  free pstVoipCodec[%d] pointer [%d] ****\n",
                           i, pstVoipCodec[i]); */
                    free(pstVoipCodec[i]);
                    pstVoipCodec[i] = NULL;
                }
                /* printf("****  free pstVoipCodec pointer [%d] ****\n",
                       pstVoipCodec); */
                free(pstVoipCodec);
                pstVoipCodec = NULL;
            }
            break;

        default:
            break;
    }

    return 1;
}

/*****************************************************************************
 *  Function Name   : IFX_CM_MgtInit
 *  Description     : This function is used by webs/snmp to Initialise the 
 *                    CM Interface
 *
 *  Input Values    :
 *  Output Values   :
 *
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/ 
PUBLIC char8  IFX_CM_MgtInit (IN uchar8 ucModuleId) 
{
  int16 ntmpfd;
  if (ucModuleId == IFX_IPC_APP_ID_WEB)
  {
    
    /* Check if Web Fifo Exists, if not Create it */ 
    if (access (IFX_IPC_WEB_FIFO, F_OK) == -1)
    {
      if (mkfifo (IFX_IPC_WEB_FIFO, IFX_IPC_FIFO_PERM))
      {
        return IFX_CM_FAIL;
      }
    }
  }
  if (ucModuleId == IFX_IPC_APP_ID_SNMP)
  {
    
    /* Check if SNMP Fifo Exists, if not Create it */ 
    if (access (IFX_IPC_SNMP_FIFO, F_OK) == -1)
    {
      if (mkfifo (IFX_IPC_SNMP_FIFO, IFX_IPC_FIFO_PERM))
      {
        return IFX_CM_FAIL;
      }
    }
  }
  
  /* Check if CM Fifo Exists, if not Create it */ 
  if (access (IFX_IPC_CM_FIFO, F_OK) == -1)
  {
    if (mkfifo (IFX_IPC_CM_FIFO, IFX_IPC_FIFO_PERM))
    {
      return IFX_CM_FAIL;
    }
  }
  ntmpfd = open (IFX_IPC_CM_FIFO, O_RDONLY | O_NDELAY);
  nCmFifoFd = open (IFX_IPC_CM_FIFO, O_WRONLY);
  
  if (nCmFifoFd < 0)
  {
    return IFX_CM_FAIL;
  }

  close (ntmpfd);
  if (ucModuleId == IFX_IPC_APP_ID_WEB)
  {
   /* Open Web fifo in blocking mode */ 
    ntmpfd = open (IFX_IPC_WEB_FIFO, O_RDONLY | O_NDELAY);
    ntmpfd = open (IFX_IPC_WEB_FIFO, O_WRONLY);
    nMgtFifoFd = open (IFX_IPC_WEB_FIFO, O_RDONLY);
    close (ntmpfd);
    if (nMgtFifoFd < 0)
    {
      return IFX_CM_FAIL;
    }
    nMgtId = IFX_IPC_APP_ID_WEB;
  }
  
  if (ucModuleId == IFX_IPC_APP_ID_SNMP)
  {
    /* Open SNMP fifo in blocking mode */ 
    ntmpfd = open (IFX_IPC_SNMP_FIFO, O_RDONLY | O_NDELAY);
    ntmpfd = open (IFX_IPC_SNMP_FIFO, O_WRONLY);
    nMgtFifoFd = open (IFX_IPC_SNMP_FIFO, O_RDONLY);
    close (ntmpfd);
    if (nMgtFifoFd < 0)
    {
      return IFX_CM_FAIL;
    }
    nMgtId = IFX_IPC_APP_ID_SNMP;
  }
  memset (&xPrevMibMsg, 0x00, sizeof(IFX_MIB_MSG_S));
  return IFX_CM_SUCCESS;
}


/*****************************************************************************
 * man  Function Name   : IFX_CM_MgtShut
 *  Description     : This function is called by webs/snmp to shutdown the
 *                    CM Interface
 *
 *  Input Values    :
 *  Output Values   :
 *
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/ 
PUBLIC char8  IFX_CM_MgtShut () 
{
  ifx_MIB_free_Dalloc();
  close (nCmFifoFd);
  close (nMgtFifoFd);
  return IFX_CM_SUCCESS;
}


/*****************************************************************************
 *  Function Name   : IFX_CM_RecvDataFromCM 
 *  Description     : This function is used by webs to Get Values from CM 
 *                    
 *                   
 *  Input Values    : CM Message 
 *  Output Values   : CM Message 
 *                    
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/ 
PUBLIC char8  IFX_CM_RecvDataFromCM (IN_OUT x_IFX_CM_Msg * pxCmMsg) 
{
  int16 nBytes = 0, ncount = 0, nindex = 0, nflag = 0;
  x_IFX_IPC_Msg xTxMsg, xRxMsg;
  
  /* format the message */ 
  memcpy (&xTxMsg.acMsg, pxCmMsg, sizeof (x_IFX_CM_Msg));
  xTxMsg.xHdr.ucFrom = nMgtId;
  xTxMsg.xHdr.ucTo = IFX_IPC_APP_ID_CM;
  xTxMsg.xHdr.unMsgSize = sizeof (x_IFX_CM_Msg);

  /* Check the validity of Message Type */ 
  if (pxCmMsg->ucMsgType == IFX_CM_GET_REQ)
  {
    /* Write Request to CM Fifo */ 
    nBytes =
    write (nCmFifoFd, (char8 *) & xTxMsg,
           sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE);
    if (nBytes != sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE)
      
    {
      return IFX_CM_FAIL;
    }
    
    /* Block in the WS Fifo */ 
    nBytes =
    read (nMgtFifoFd, (char8 *) & xRxMsg,
          sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE);
    if (nBytes < 0)
    {
      return IFX_CM_FAIL;
    }
    memcpy ((char8 *) pxCmMsg, (char8 *) &xRxMsg.acMsg, sizeof (x_IFX_CM_Msg));
    
    /* Check the response */ 
    if ((pxCmMsg->ucMsgType == IFX_CM_GET_RSP) || 
        (pxCmMsg->ucMsgType == IFX_CM_GET_ERR_RSP))
    {
      if (pxCmMsg->ucInfoType == IFX_CM_APP_ADDR_ENTRY)
      {
        for (ncount = 0; ncount < 105; ncount += 5)
        {
          nindex = pxCmMsg->uxData.xAddrBookCfg.ucNoOfEntries;
          memcpy (&axPBEntry[ncount], &pxCmMsg->uxData.xAddrBookCfg.axAddrBookEntry,
                  sizeof (x_IFX_CM_AppAddressBookEntry) * nindex);
		  
          if (nindex < 5)
          {
            nNoOfPbEntries = ncount + nindex;
            nflag = 1;
            break;
          }
          nBytes =
            read (nMgtFifoFd, (char8 *) & xRxMsg,
                   sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE);
		  
          memcpy ((char8 *) pxCmMsg, (char8 *) &xRxMsg.acMsg,
                    sizeof (x_IFX_CM_Msg));
          if (nBytes < 0)
          {
            return IFX_CM_FAIL;
          }
        }
        if (nNoOfPbEntries == 99 && !nflag)
          nNoOfPbEntries++;
      }
    }
    else
    {
      return IFX_CM_FAIL;
    }
  }
  else
  {
    return IFX_CM_FAIL;
  }
  return IFX_CM_SUCCESS;
}


/*****************************************************************************
 *  Function Name   : IFX_CM_SendDataToCM
 *  Description     : This function is used by webs to configure parameters 
 *                    in the CM 
 *
 *  Input Values    : CM Message 
 *  Output Values   : CM Message 
 *                     
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/ 
PUBLIC char8  IFX_CM_SendDataToCM (IN_OUT x_IFX_CM_Msg * pxCmMsg) 
{
  int16 nBytes = 0;
  x_IFX_IPC_Msg xTxMsg, xRxMsg;
  
 /* format the message */ 
  memcpy (&xTxMsg.acMsg, pxCmMsg, sizeof (x_IFX_CM_Msg));
  xTxMsg.xHdr.ucFrom = nMgtId;
  xTxMsg.xHdr.ucTo = IFX_IPC_APP_ID_CM;
  xTxMsg.xHdr.unMsgSize = sizeof (x_IFX_CM_Msg);
  
  /* Check the validity of Message Type */ 
  if (pxCmMsg->ucMsgType == IFX_CM_SET_REQ)
  {
      /* Write Request to CM Fifo */ 
    nBytes =
    write (nCmFifoFd, (char8 *) & xTxMsg,
           sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE);
    if (nBytes != sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE)
    {
      return IFX_CM_FAIL;
    }
    
    /* Block in the WS Fifo */ 
    nBytes =
    read (nMgtFifoFd, (char8 *) & xRxMsg,
          sizeof (x_IFX_CM_Msg) + IFX_IPC_HDR_SIZE);
    if (nBytes < 0)
    {
      return IFX_CM_FAIL;
    }
    
    /* To check if this would work.  Trying to avoid memcpy here */ 
    memcpy ((char8 *) pxCmMsg, (char8 *) &xRxMsg.acMsg, sizeof (x_IFX_CM_Msg));
    
    /* Check the response */ 
    if (pxCmMsg->ucMsgType == IFX_CM_SET_RSP)
    {
      return IFX_CM_SUCCESS;
    }
    else if (pxCmMsg->ucMsgType == IFX_CM_SET_ERR_RSP)
    {
      return IFX_CM_SUCCESS;
    }
    else
    {
      return IFX_CM_FAIL;
    }
  }
  else
  {
    return IFX_CM_FAIL;
  }
  return IFX_CM_SUCCESS;
}

int
ifx_helper(netsnmp_mib_handler *handler,
           netsnmp_handler_registration *reginfo,
           netsnmp_agent_request_info *reqinfo,
           netsnmp_request_info *requests,
           netsnmp_helper_info *winfo)
{
    u_char   *old_data;

    switch (reqinfo->mode) {
        /*
         * data requests 
         */
        case MODE_GET:
            snmp_set_var_typed_value(requests->requestvb,
                                 winfo->type,
                                 winfo->data,
                                 winfo->data_size);
            break;

        /*
         * SET requests.  Should only get here if registered RWRITE 
         */
        case MODE_SET_RESERVE1:
            if (requests->requestvb->type != winfo->type)
                netsnmp_set_request_error(reqinfo, requests,
                                      SNMP_ERR_WRONGTYPE);
            
            if (((winfo->flags & HELPER_CHECK_MAX) &&
                requests->requestvb->val_len > winfo->max_size) ||
                ((winfo->flags & HELPER_FIXED_SIZE) &&
                requests->requestvb->val_len != winfo->max_size))
                netsnmp_set_request_error(reqinfo, requests,
                                      SNMP_ERR_WRONGLENGTH);
            else                          
            if ( (winfo->flags & HELPER_CHECK_MAX_MIN) &&
                 ((*(requests->requestvb->val.integer) > winfo->max_size) ||
                  (*(requests->requestvb->val.integer) < winfo->min_size)) )
                netsnmp_set_request_error(reqinfo, requests,
                                      SNMP_ERR_WRONGVALUE);
            break;

        case MODE_SET_RESERVE2:
            /*
             * store old info for undo later 
             */
            memdup(&old_data, (u_char *) winfo->data, winfo->data_size);
            if (old_data == NULL) {
                netsnmp_set_request_error(reqinfo, requests,
                                      SNMP_ERR_RESOURCEUNAVAILABLE);
                return SNMP_ERR_NOERROR;
            }
            netsnmp_request_add_list_data(requests,
                                      netsnmp_create_data_list
                                      (IFX_MIB_SET_NAME, old_data, free));
            break;

        case MODE_SET_ACTION:
            /*
             * update current 
             */
            if (winfo->flags & HELPER_MEMSET)
                memset(winfo->data, 0, winfo->data_size);
            memcpy(winfo->data, (void *)requests->requestvb->val.string,
                                    requests->requestvb->val_len);
            break;

        case MODE_SET_COMMIT:
        case MODE_SET_FREE:
            break;

        case MODE_SET_UNDO:
            memcpy(winfo->data,
               netsnmp_request_get_list_data(requests, IFX_MIB_SET_NAME),
               winfo->data_size);
            break;

        default:
            return SNMP_ERR_GENERR;
    }
    return SNMP_ERR_NOERROR;
}
