/************************************************************************
 *
 *    Copyright (c) 2005
 *    Infineon Technologies AG
 *    St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 *    This document contains proprietary information belonging to Infineon
 *    Technologies AG. Passing on and copying of this document, and communication
 *    of its contents is not permitted without prior written authorisation.
 *
 *    --------------------------------------------------------------------
 *    
 *        Project : INCA-IP
 *        Block   : RTP MIB
 *        Creator : Sharon Tseng
 *        File    : rtpSessionTable.c
 *        Abstract: 
 *                  
 *        Date    : 12-May-2005
 *        Ref. Doc: RFC 2959
 *    
 *    Modification History:
 *           By              Date     Ver.   Modification Description
 *           --------------- -------- -----  -----------------------------
 *
 ************************************************************************/

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.iterate_access.conf,v 1.6 2003/09/29 22:30:36 hardaker Exp $
 */
/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "rtpSessionTable.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/
static netsnmp_oid_stash_node *undoStorage = NULL;
static netsnmp_oid_stash_node *commitStorage = NULL;

struct undoInfo {
    void           *ptr;
    size_t          len;
};

struct commitInfo {
    void           *data_context;
    int             have_committed;
    int             new_row;
};

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/
void
rtpSessionTable_free_undoInfo(void *vptr)
{
    struct undoInfo *ui = vptr;
    if (!ui)
        return;
    SNMP_FREE(ui->ptr);
    SNMP_FREE(ui);
}

/** Initialize the rtpSessionTable table by defining its contents and how it's structured */
void
initialize_table_rtpSessionTable(void)
{
    static oid      rtpSessionTable_oid[] = { 1, 3, 6, 1, 2, 1, 87, 1, 3 };
    netsnmp_table_registration_info *table_info;
    netsnmp_handler_registration *my_handler;
    netsnmp_iterator_info *iinfo;

    /** create the table registration information structures */
    table_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_registration_info);
    iinfo = SNMP_MALLOC_TYPEDEF(netsnmp_iterator_info);

    /** if your table is read only, it's easiest to change the
        HANDLER_CAN_RWRITE definition below to HANDLER_CAN_RONLY */
    my_handler = netsnmp_create_handler_registration("rtpSessionTable",
                                                     rtpSessionTable_handler,
                                                     rtpSessionTable_oid,
                                                     OID_LENGTH(rtpSessionTable_oid),
                                                     HANDLER_CAN_RONLY);

    if (!my_handler || !table_info || !iinfo)
    {

#ifdef IIP_MEM_LEAK_FIX
         if (my_handler)
            SNMP_FREE(my_handler);
         if (table_info)
            SNMP_FREE(table_info);
          if (iinfo)
            SNMP_FREE(iinfo);
#endif
        snmp_log(LOG_ERR,\
                 "malloc failed in initialize_table_rtpSessionTable\n");
        return; /** Serious error. */
    }

    /* printf("**** initialize_table_rtpSessionTable ****\n"); */

    /***************************************************
     * Setting up the table's definition
     */
    netsnmp_table_helper_add_indexes(table_info, ASN_INTEGER,
                                               /** index: rtpSessionIndex */
                                     0);

    /** Define the minimum and maximum accessible columns.  This
        optimizes retrival. */
    table_info->min_column = COLUMN_RTPSESSIONDOMAIN;
    table_info->max_column = COLUMN_RTPSESSIONROWSTATUS;

    /** iterator access routines */
    iinfo->get_first_data_point = rtpSessionTable_get_first_data_point;
    iinfo->get_next_data_point = rtpSessionTable_get_next_data_point;

    /** you may wish to set these as well */
#ifdef MAYBE_USE_THESE
    iinfo->make_data_context = rtpSessionTable_context_convert_function;
    iinfo->free_data_context = rtpSessionTable_data_free;

    /** pick *only* one of these if you use them */
    iinfo->free_loop_context = rtpSessionTable_loop_free;
    iinfo->free_loop_context_at_end = rtpSessionTable_loop_free;
#endif

    /** tie the two structures together */
    iinfo->table_reginfo = table_info;

    /***************************************************
     * registering the table with the master agent
     */
    DEBUGMSGTL(("initialize_table_rtpSessionTable",
                "Registering table rtpSessionTable as a table iterator\n"));
    netsnmp_register_table_iterator(my_handler, iinfo);
}

/** Initializes the rtpSessionTable module */
void
init_rtpSessionTable(void)
{
    /** here we initialize all the tables we're planning on supporting */
    initialize_table_rtpSessionTable();
}

/** handles requests for the rtpSessionTable table, if anything else needs to be done */
int
rtpSessionTable_handler(netsnmp_mib_handler *handler,
                        netsnmp_handler_registration *reginfo,
                        netsnmp_agent_request_info *reqinfo,
                        netsnmp_request_info *requests)
{
    netsnmp_request_info        *request;
    netsnmp_table_request_info  *table_info;
    netsnmp_variable_list       *var;
    struct commitInfo           *ci = NULL;
    void                        *data_context = NULL;
    oid                         *suffix;
    size_t                      suffix_len;

    for (request = requests; request; request = request->next)
    {
        /** column and row index encoded portion */
        var = request->requestvb;
        suffix = var->name + reginfo->rootoid_len + 1;
        suffix_len = var->name_length - (reginfo->rootoid_len + 1);
        if (request->processed != 0)
            continue;

        switch (reqinfo->mode)
        {
        case MODE_GET:
            data_context = netsnmp_extract_iterator_context(request);
            if (data_context == NULL)
            {
                if (reqinfo->mode == MODE_GET)
                {
                    netsnmp_set_request_error(reqinfo, request,
                                              SNMP_NOSUCHINSTANCE);
                    continue;
                }
            }
            break;

        case MODE_SET_RESERVE1:
            data_context = netsnmp_extract_iterator_context(request);
            if (data_context == NULL)
            {
                netsnmp_set_request_error(reqinfo, request,
                                          SNMP_ERR_NOCREATION);
                continue;
            }
            break;

        default:               /* == the other SET modes */
            ci = netsnmp_oid_stash_get_data(commitStorage,
                                            suffix + 1, suffix_len - 1);
            break;

        }

        /** extracts the information about the table from the request */
        table_info = netsnmp_extract_table_info(request);
        /** table_info->colnum contains the column number requested */
        /** table_info->indexes contains a linked list of snmp variable
           bindings for the indexes of the table.  Values in the list
           have been set corresponding to the indexes of the
           request */
        if (table_info == NULL)
        {
            continue;
        }

        switch (reqinfo->mode)
        {
        case MODE_GET:
	            switch (table_info->colnum)
	            {
            case COLUMN_RTPSESSIONDOMAIN:
                {
                    long            *retval;
                    size_t          retval_len = 0;
		            retval = get_rtpSessionDomain(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_OBJECT_ID,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONREMADDR:
                {
                    unsigned char   *retval;
                    size_t          retval_len = 0;
		            retval = get_rtpSessionRemAddr(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_OCTET_STR,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONLOCADDR:
                {
                    unsigned char   *retval;
                    size_t          retval_len = 0;
		            retval = get_rtpSessionLocAddr(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_OCTET_STR,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONIFINDEX:
                {
                    long           *retval;
                    size_t          retval_len = 0;
		            retval = get_rtpSessionIfIndex(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONSENDERJOINS:
                {
                    unsigned int    *retval;
                    size_t          retval_len = 0;
		            retval = get_rtpSessionSenderJoins(data_context,
                                                  &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONRECEIVERJOINS:
                {
                    unsigned int   *retval;
                    size_t          retval_len = 0;
		                    retval = get_rtpSessionReceiverJoins(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONBYES:
                {
                    unsigned int   *retval;
                    size_t          retval_len = 0;
                    retval = get_rtpSessionByes(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_COUNTER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONSTARTTIME:
                {
                    unsigned int   *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_rtpSessionStartTime(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_TIMETICKS,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONMONITOR:
                {
                    long           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_rtpSessionMonitor(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_RTPSESSIONROWSTATUS:
                {
                    long           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_rtpSessionRowStatus(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            default:
                /** We shouldn't get here */
                snmp_log(LOG_ERR,
                         "problem encountered in rtpSessionTable_handler: unknown column\n");
            }
            break;

#ifdef IFX_RTP_WRITABLE
        case MODE_SET_RESERVE1:
            ci = netsnmp_oid_stash_get_data(commitStorage,
                                            suffix + 1, suffix_len - 1);

            if (!ci) {
                    /** create the commit storage info */
                ci = SNMP_MALLOC_STRUCT(commitInfo);
                if (!data_context) {
                    ci->data_context =
                        rtpSessionTable_create_data_context(table_info->
                                                            indexes);
                    ci->new_row = 1;
                } else {
                    ci->data_context = data_context;
                }
                netsnmp_oid_stash_add_data(&commitStorage,
                                           suffix + 1, suffix_len - 1, ci);
            }
            break;

        case MODE_SET_RESERVE2:
            switch (table_info->colnum) {
            case COLUMN_RTPSESSIONDOMAIN:
                {
                    long            *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_rtpSessionDomain(ci->data_context,
                                             &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret = check_rtpSessionDomain(var->type,
                                                 (long *) var->val.string,
                                                 var->val_len, retval,
                                                 retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        rtpSessionTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_RTPSESSIONREMADDR:
                {
                    unsigned char   *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_rtpSessionRemAddr(ci->data_context,
                                              &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret = check_rtpSessionRemAddr(var->type,
                                                  (unsigned char *) var->val.string,
                                                  var->val_len, retval,
                                                  retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        rtpSessionTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_RTPSESSIONIFINDEX:
                {
                    long           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_rtpSessionIfIndex(ci->data_context,
                                              &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret = check_rtpSessionIfIndex(var->type,
                                                  (long *) var->val.string,
                                                  var->val_len, retval,
                                                  retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        rtpSessionTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_RTPSESSIONROWSTATUS:
                {
                    long           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_rtpSessionRowStatus(ci->data_context,
                                                &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_rtpSessionRowStatus(var->type,
                                                  (long *) var->val.string,
                                                  var->val_len, retval,
                                                  retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        rtpSessionTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;

            default:
                netsnmp_set_request_error(reqinfo, request,
                                          SNMP_ERR_NOTWRITABLE);
                break;
            }
            break;

        case MODE_SET_ACTION:
            /** save a variable copy */
            switch (table_info->colnum) {
            case COLUMN_RTPSESSIONDOMAIN:
                {
                    int             ret;
                    ret = set_rtpSessionDomain(ci->data_context,
                                               (long *) var->val.string,
                                               var->val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_RTPSESSIONREMADDR:
                {
                    int             ret;
                    ret = set_rtpSessionRemAddr(ci->data_context,
                                                (unsigned char *) var->val.string,
                                                var->val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_RTPSESSIONIFINDEX:
                {
                    int             ret;
                    ret = set_rtpSessionIfIndex(ci->data_context,
                                                (long *) var->val.string,
                                                var->val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_RTPSESSIONROWSTATUS:
                {
                    int             ret;
                    ret = set_rtpSessionRowStatus(ci->data_context,
                                                  (long *) var->val.string,
                                                  var->val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                    if (*var->val.integer == RS_DESTROY) {
                        ci->new_row = -1;
                    }
                }
                break;
            }
            break;

        case MODE_SET_COMMIT:
            if (!ci->have_committed) {
                    /** do this once per row only */
                rtpSessionTable_commit_row(&ci->data_context, ci->new_row);
                ci->have_committed = 1;
            }
            break;

        case MODE_SET_UNDO:
             /** save a variable copy */
            switch (table_info->colnum) {
            case COLUMN_RTPSESSIONDOMAIN:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_rtpSessionDomain(ci->data_context, ui->ptr,
                                             ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_RTPSESSIONREMADDR:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_rtpSessionRemAddr(ci->data_context, ui->ptr,
                                              ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_RTPSESSIONIFINDEX:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_rtpSessionIfIndex(ci->data_context, ui->ptr,
                                              ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_RTPSESSIONROWSTATUS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_rtpSessionRowStatus(ci->data_context, ui->ptr,
                                                ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            }
            break;

        case MODE_SET_FREE:
            break;
#endif /* IFX_RTP_WRITABLE */

        default:
            snmp_log(LOG_ERR,
                     "problem encountered in rtpSessionTable_handler: unsupported mode\n");
        }
    }

#ifdef IFX_RTP_WRITABLE
    /** clean up after all requset processing has ended */
    switch (reqinfo->mode) {
    case MODE_SET_UNDO:
    case MODE_SET_FREE:
    case MODE_SET_COMMIT:
        /** clear out the undo cache */
        netsnmp_oid_stash_free(&undoStorage,
                               rtpSessionTable_free_undoInfo);
        netsnmp_oid_stash_free(&commitStorage, netsnmp_oid_stash_no_free);
    }
#endif /* IFX_RTP_WRITABLE */

    return SNMP_ERR_NOERROR;
}
